DeflectorShield = {}
DeflectorShieldSaved = {
	FrameScale = 1,
	FrameX = 0,
	FrameY = 40,
	
	RefreshRate = 4
}
DeflectorShieldCharSaved = {
	ShowFrame = 1
}

local VarsLoaded

local GUID
local StopRefreshing = 0
local NextRefresh = 0.25

local ShieldBuffs = {
	[77535] = { -- Blood Shield (Death Knight - Blood mastery)
		Value = 1,
		r = 0.77,
		g = 0.12,
		b = 0.23
	},
	[47753] = { -- Divine Aegis (Priest - Discipline passive)
		Value = 1,
		r = 0.75,
		g = 0.75,
		b = 0.75
	},
	[11426] = { -- Ice Barrier (Mage - castable talent)
		Value = 1,
		r = 0.41,
		g = 0.80,
		b = 0.94
	},
	[86273] = { -- Illuminated Healing (Paladin - Holy mastery)
		Value = 1,
		r = 0.96,
		g = 0.55,
		b = 0.73
	},
	[1463] = { -- Incanter's Ward (Mage - castable talent)
		Value = 1,
		r = 1.00,
		g = 1.00,
		b = 0.00
	},
	[17] = { -- Power Word: Shield (Priest - castable ability)
		Value = 1,
		r = 1.00,
		g = 1.00,
		b = 1.00
	},
	[65148] = { -- Sacred Shield (Paladin - castable talent)
		Value = 1,
		r = 1.00,
		g = 0.70,
		b = 1.00
	},
	[108416] = { -- Sacrificial Pact (Warlock - castable talent)
		Value = 1,
		r = 0.58,
		g = 0.51,
		b = 0.79
	},
	["112048"] = { -- Shield Barrier (Warrior - Protection ability)
		Value = 1,
		r = 0.78,
		g = 0.61,
		b = 0.43
	}
}

local Bars = {}

local FrameIsMoving

function DeflectorShield.Refresh()
	local Now = GetTime()
	local Absorption = 0
	
	local Bar = 1
	local Buff = 1
	local Duration, Expires, SpellID, _
	local Values = {}
	_, _, _, _, _, Duration, Expires, _, _, _, SpellID, _, _, Values[1], Values[2], Values[3] = UnitBuff("player", Buff)
	while SpellID do
		if ShieldBuffs[SpellID] then
			Absorption = Absorption + Values[ShieldBuffs[SpellID].Value]
			
			local TimeLeft = Expires - Now
			local PercentLeft = ceil((TimeLeft / Duration) * 100)
			local x = PercentLeft + 5
			
			if not Bars[Bar] then
				Bars[Bar] = {}
				
				Bars[Bar].Frame = CreateFrame("Frame", nil, DeflectorShieldFrame)
				
				Bars[Bar].Frame:SetWidth(3)
				Bars[Bar].Frame:SetHeight(19)

				Bars[Bar].Texture = Bars[Bar].Frame:CreateTexture("ARTWORK")
				Bars[Bar].Texture:SetAllPoints()
				Bars[Bar].Texture:SetAlpha(1.0)
			end
			
			Bars[Bar].Frame:ClearAllPoints()
			Bars[Bar].Frame:SetPoint("CENTER", "DeflectorShieldFrame", "LEFT", x, 0)
			Bars[Bar].Texture:SetTexture(ShieldBuffs[SpellID].r, ShieldBuffs[SpellID].g, ShieldBuffs[SpellID].b)			
			if not Bars[Bar].Frame:IsShown() then Bars[Bar].Frame:Show() end
			
			Bar = Bar + 1
			
			StopRefreshing = Now + 5
		end
				
		Buff = Buff + 1
		_, _, _, _, _, Duration, Expires, _, _, _, SpellID, _, _, Values[1], Values[2], Values[3] = UnitBuff("player", Buff)
	end
	
	while Bars[Bar] do
		if Bars[Bar].Frame:IsShown() then Bars[Bar].Frame:Hide() end
		Bar = Bar + 1
	end

	DeflectorShieldFrameAbsorption:SetText(Absorption)
	
	NextRefresh = 1 / DeflectorShieldSaved.RefreshRate
end

function DeflectorShield.FrameOnLoad(Frame)
	Frame:RegisterEvent("ADDON_LOADED")
	Frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function DeflectorShield.FrameOnEvent(Frame, Event, ...)
	if Event == "ADDON_LOADED" then
		VarsLoaded = true
		
		GUID = UnitGUID("player")
		
		if DeflectorShieldCharSaved.ShowFrame == 1 then
			Frame:SetScale(DeflectorShieldSaved.FrameScale)
			Frame:ClearAllPoints()
			Frame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", DeflectorShieldSaved.FrameX, DeflectorShieldSaved.FrameY)

			Frame:Show()
		end		
	end
	if not VarsLoaded then return end
	
	if Event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local _, _, _, _, _, _, _, DestGUID, _, _, _, _ = ...
		if DestGUID == GUID then
			StopRefreshing = GetTime() + 5
			DeflectorShield.Refresh()
		end
	end
end

function DeflectorShield.FrameOnUpdate(Frame, Elapsed)
	if GetTime() >= StopRefreshing then return end

	NextRefresh = NextRefresh - Elapsed	
	if NextRefresh <= 0 then DeflectorShield.Refresh() end
end

function DeflectorShield.FrameOnMouseDown(Frame, Button)
	if (Button == "LeftButton" or Button == "RightButton") and IsShiftKeyDown() and not FrameIsMoving then
		Frame:StartMoving()
		FrameIsMoving = true
	end
end

function DeflectorShield.FrameOnMouseUp(Frame, Button)
	if (Button == "LeftButton" or Button == "RightButton") then
		if FrameIsMoving then
			Frame:StopMovingOrSizing()
			FrameIsMoving = false
			
			DeflectorShieldSaved.FrameX = Frame:GetLeft()
			DeflectorShieldSaved.FrameY = Frame:GetBottom()
		end
	end
end

function DeflectorShield.FrameOnMouseWheel(Frame, Direction)
	if IsShiftKeyDown() then
		local NewScale = DeflectorShieldSaved.FrameScale + (Direction / 10)
		if NewScale > 0 then
			Frame:SetScale(NewScale)
			
			DeflectorShieldSaved.FrameScale = NewScale
			DeflectorShieldSaved.FrameX = Frame:GetLeft()
			DeflectorShieldSaved.FrameY = Frame:GetBottom()
		end
	end
end

function DeflectorShield.FrameOnHide(Frame)
	if FrameIsMoving then
		Frame:StopMovingOrSizing()
		FrameIsMoving = false
		
		DeflectorShieldSaved.FrameX = Frame:GetLeft()
		DeflectorShieldSaved.FrameY = Frame:GetBottom()
	end
end

function DeflectorShield.SlashCmds(Msg, EditBox)
	local lMsg
	if Msg then lMsg = string.lower(Msg) end
	
	if string.sub(lMsg, 1, 1) == "f" then
		if DeflectorShieldCharSaved.ShowFrame == 1 then
			DeflectorShieldFrame:Hide()
			
			DeflectorShieldCharSaved.ShowFrame = 0
		else
			DeflectorShieldFrame:SetScale(DeflectorShieldSaved.FrameScale)
			DeflectorShieldFrame:ClearAllPoints()
			DeflectorShieldFrame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", DeflectorShieldSaved.FrameX, DeflectorShieldSaved.FrameY)

			DeflectorShieldFrame:Show()

			DeflectorShieldCharSaved.ShowFrame = 1
		end
	elseif string.sub(lMsg, 1, 1) == "s" then
		local NewScale = tonumber(string.sub(Msg, 3))
		if NewScale > 0 then
			NewScale = NewScale / 100
			
			DeflectorShieldFrame:SetScale(NewScale)
		
			DeflectorShieldSaved.FrameScale = NewScale
			DeflectorShieldSaved.FrameX = DeflectorShieldFrame:GetLeft()
			DeflectorShieldSaved.FrameY = DeflectorShieldFrame:GetBottom()
		end
		
		print("|cff0099ffWyeSoft Deflector Shield:|r Frame scale: " .. (DeflectorShieldSaved.FrameScale * 100) .. "%")
	elseif string.sub(lMsg, 1, 5) == "reset" then
		DeflectorShieldFrame:SetScale(1)
		DeflectorShieldFrame:ClearAllPoints()
		DeflectorShieldFrame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", 0, 40)
		
		DeflectorShieldSaved.FrameScale = 1
		DeflectorShieldSaved.FrameX = 0
		DeflectorShieldSaved.FrameY = 40
	elseif string.sub(lMsg, 1, 1) == "r" then
		local NewRate = tonumber(string.sub(Msg,3))
		if NewRate > 0 then
			DeflectorShieldSaved.RefreshRate = NewRate
		end
		
		print("|cff0099ffWyeSoft Deflector Shield:|r Refresh rate: " .. DeflectorShieldSaved.RefreshRate .. " refreshes per second")
	else
		print("|cff0099ffWyeSoft Deflector Shield:|r Available commands:")
		print("|cffffff00/SHIELD F|r  toggles display of Deflector Shield frame.  This setting is saved separately for each character.")
		print("/SHIELD R <Rate>|r  changes the refresh rate to Rate per second.  Example /SHIELD R 4  (4 refreshes per second, the default).")
		print("/SHIELD S <Scale>|r  resizes the Deflector Shield frame to the given percentage of its default size.  Example /SHIELD S 100  (do not enter a % symbol).")
		print("|cffffff00/SHIELD RESET|r  resets size and position of Deflector Shield frame.")
		print("|cffffff00/SHIELD H|r  lists Deflector Shield's slash commands.")
	end
end

SLASH_DEFLECTORSHIELD1 = "/shield"
SLASH_DEFLECTORSHIELD2 = "/deflector"
SLASH_DEFLECTORSHIELD3 = "/deflectorshield"
SLASH_DEFLECTORSHIELD4 = "/wyesoftdeflectorshield"
SlashCmdList["DEFLECTORSHIELD"] = DeflectorShield.SlashCmds
